<?php
/*
Plugin Name: 	Social Box
Plugin URI: 	http://codecanyon.net/item/socialbox-social-wordpress-widget/627127
Description: 	Adds a super easy Social Box which displays the numbers of Twitter Followers, Facebook Fans and Youtube Channel and Feed Subscribers
Version: 		1.1.1
Author: 		JonasDoebertin
Author URI: 	http://codecanyon.net/user/JonasDoebertin
License: 		Sold exclusively on CodeCanyon
*/

if(!class_exists('SocialBox') and !class_exists('SocialBoxWidget')){
	
	class SocialBox{
		
		/**
		 * The plugins slug
		 */
		const SLUG = 'socialbox';
		
		/**
		 * The plugins current version number
		 */
		const VERSION = '1.1.1';
		
		/**
		 * Facebook API Base URL
		 */
		const FACEBOOK_BASE = "http://graph.facebook.com/";
		
		/**
		 * Twitter API Base URL
		 */
		const TWITTER_BASE = "http://api.twitter.com/1/users/show.json?skip_status=true&screen_name=";
		
		/**
		 * YouTube API Base URL
		 */
		const YOUTUBE_BASE ="http://gdata.youtube.com/feeds/api/users/";
		
		/**
		 * Feedburner API Base URL
		 */
		const FEEDBURNER_BASE = "http://feedburner.google.com/api/awareness/1.0/GetFeedData?uri=";
		
		/**
		 * Update check script URL
		 */
		const UPDATE_BASE = "http://updates.jonasdoebertin.net/";
		
		/**
		 * Create an instance of the plugin
		 */
		public function __construct(){
			
			/* Register / unregister cron events */
			add_filter('cron_schedules', array($this, 'addCronSchedule'));
			add_action(self::SLUG . '_fetch', array($this, 'doFetch'));
			add_action(self::SLUG . '_update', array($this, 'doUpdateCheck'));
			register_activation_hook(__FILE__, array($this, 'activatePlugin'));
			register_deactivation_hook(__FILE__, array($this, 'deactivatePlugin'));
			
			/* If an update is available, then notify */
			if(is_admin()){
				$info = get_option(self::SLUG . '_update', null);
				if(is_array($info) and $info['update']){
					add_action('admin_notices', array($this, 'addAdminNotice'));
				}	
			}
			
			/* Register Widget */
			add_action('widgets_init', array($this, 'registerWidget'));
			
			/* Register and load textdomain */
			load_plugin_textdomain(self::SLUG, null, basename(dirname(__FILE__)) . '/languages/');
			
			/* Register Widget Style */
			if(!is_admin()){
				add_action('wp_print_styles', array($this, 'registerStyle'));
			}			
			
		}
		
		/**
		 * Inject a custom cron schedule timeframe
		 *
		 * @param Array $schedules Preexisting cron schedule timeframes
		 * @return Array Extended cron schedule timeframes
		 */
		public function addCronSchedule($schedules){
			
			$schedules['threehourly'] = array(
				'interval' => 10800,
				'display' => __('Every Three Hours')
			);
			return $schedules;
			
		}
		
		/**
		 * Refresh the social data for each SocialBox widget.
		 *
		 * This will be executed as scheduled cron event
		 */
		public function doFetch(){
			
			/* Abort if no widget has been set up */
			$opts = get_option(self::SLUG . '_options', null);
			if(!is_array($opts)){
				return;
			}
			
			/* Call static refresh function for each widget */
			foreach($opts as $widgetId => $widget){
				SocialBox::refresh($widgetId);
			}
			
		}
		
		/**
		 * Fetch update information and store it to the database
		 *
		 * This will be executed as scheduled cron event
		 */
		public function doUpdateCheck(){
			
			$query = array(
				'slug' => self::SLUG,
				'version' => self::VERSION
			);
			$url = self::UPDATE_BASE . '?' . http_build_query($query);
			
			$result = wp_remote_get($url);
			
			/* Check for unsuccessful http requests */
			if(is_wp_error($result) or (wp_remote_retrieve_response_code($result) != 200)){
				$info = array(
					'update' => false
				);
			}
							
			/* Check for incorrect data */
			$info = unserialize(wp_remote_retrieve_body($result));
			if(!is_array($info) or isset($info['error']) or !isset($info['update'])){
				$info = array(
					'update' => false
				);
			}
			
			update_option(self::SLUG . '_update', $info);
			
		}
		
		/**
		 * Register options and cron schedules
		 *
		 * Will be run through register_activation_hook()
		 */
		public function activatePlugin(){
			
			/* Add options */
			add_option(self::SLUG . '_update', null);
			add_option(self::SLUG . '_options', null);
			
			/* Register cron events */
			wp_schedule_event(time(), 'threehourly', self::SLUG . '_fetch');
			wp_schedule_event(time(), 'hourly', self::SLUG . '_update');
			
		}
		
		/**
		 * Deregister options and cron schedules
		 *
		 * Will be run through register_deactivation_hook
		 */
		public function deactivatePlugin(){
			
			/* Delete options */
			delete_option(self::SLUG . '_update');
			//delete_option(self::SLUG . '_options');
			
			/* Deregister cron events */
			wp_clear_scheduled_hook(self::SLUG . '_fetch');
			wp_clear_scheduled_hook(self::SLUG . '_update');
			
		}
		
		/**
		 * Output an update notice in admin screens
		 *
		 * Will be run "admin_notices" action
		 */
		public function addAdminNotice(){
			
			$info = get_option(self::SLUG . '_update');
			
			?>
			
			<div id="message" class="updated">
				<p>
					<strong><?php _e('SocialBox update available!', self::SLUG); ?></strong>
					<?php printf(__('New Version: %s (%s). Info and download at the <a href="%s">plugin page</a>.', self::SLUG), $info['version'], $info['date'], $info['link']); ?>
				</p>
			</div>
			
			<?php
			
		}
		
		/**
		 * Register the Widget
		 *
		 * Will be run in "widgets_init" action
		 */
		public function registerWidget(){
			
			register_widget('SocialBoxWidget');
			
		}
		
		/**
		 * Register the widgets stylesheet
		 *
		 * Will be run in "wp_print_styles" action and only on frontend pages
		 */
		public function registerStyle(){
			
			/* Register style */
			wp_register_style(self::SLUG, plugins_url('css/socialbox.css', __FILE__), array(), self::VERSION, 'screen');
			
			/* Enqueue style */
			wp_enqueue_style(self::SLUG);
			
		}
		
		/**
		 * Refresh data for given Widget
		 *
		 * Will be called after updating a Widgets settings or by "SocialBox::doFetch()"
		 */
		public static function refresh($widgetId){
			
			/* Get options if available */
			$opts = get_option(SocialBox::SLUG . '_options', null);
			if(is_array($opts) and isset($opts[$widgetId])){
				$widget = $opts[$widgetId];
			} else{
				return;
			}
			
			/* Refresh Facebook count */
			if(isset($widget['facebook_id']) and !empty($widget['facebook_id'])){
				$newFacebookCount = SocialBox::refreshFacebook($widget['facebook_id']);
				$widget['facebook_count'] = ((is_numeric($newFacebookCount)) ? $newFacebookCount : (($widget['set_defaults']) ? $widget['facebook_default'] : 0));
			}
			
			/* Refresh Twitter count */
			if(isset($widget['twitter_id']) and !empty($widget['twitter_id'])){
				$newTwitterCount = SocialBox::refreshTwitter($widget['twitter_id']);
				$widget['twitter_count'] = ((is_numeric($newTwitterCount)) ? $newTwitterCount : (($widget['set_defaults']) ? $widget['twitter_default'] : 0));
			}
			
			/* Refresh Youtube count */
			if(isset($widget['youtube_id']) and !empty($widget['youtube_id'])){
				$newYoutubeCount = SocialBox::refreshYoutube($widget['youtube_id']);
				$widget['youtube_count'] = ((is_numeric($newYoutubeCount)) ? $newYoutubeCount : (($widget['set_defaults']) ? $widget['youtube_default'] : 0));
			}
			
			/* Refresh Feedburner count */
			if(isset($widget['feedburner_id']) and !empty($widget['feedburner_id'])){
				$newFeedburnerCount = SocialBox::refreshFeedburner($widget['feedburner_id']);
				$widget['feedburner_count'] = ((is_numeric($newFeedburnerCount)) ? $newFeedburnerCount : (($widget['set_defaults']) ? $widget['feedburner_default'] : 0));
				if(is_numeric($newFeedburnerCount) and ($newFeedburnerCount == 0) and $widget['set_defaults']){
					$widget['feedburner_count']	= $widget['feedburner_default'];
				}
			}
			
			/* Save new data */
			$opts[$widgetId] = $widget;
			update_option(SocialBox::SLUG . '_options', $opts);
			
		}
		
		/**
		 * Get Facebook likes for given account
		 *
		 * Will be called by "SocialBox::refresh()"
		 */
		public static function refreshFacebook($facebookId){
			
			/* Fetch data */
			$result = wp_remote_get(self::FACEBOOK_BASE . $facebookId);
			
			/* Check for unsuccessful http requests */
			if(is_wp_error($result) or (wp_remote_retrieve_response_code($result) != 200)){
				return false;
			}
			
			/* Check for incorrect data */
			$data = json_decode(wp_remote_retrieve_body($result), true);
			if(!is_array($data) or isset($data['error']) or !isset($data['likes'])){
				return false;
			}
			
			/* Return like count */
			return $data['likes'];
			
		}
		
		/**
		 * Get Twitter followers for given account
		 *
		 * Will be called by "SocialBox::refresh()"
		 */
		public static function refreshTwitter($twitterId){
			
			/* Fetch data */
			$result = wp_remote_get(self::TWITTER_BASE . $twitterId);
			
			/* Check for unsuccessful http requests */
			if(is_wp_error($result) or (wp_remote_retrieve_response_code($result) != 200)){
				return false;
			}
			
			/* Check for incorrect data */
			$data = json_decode(wp_remote_retrieve_body($result), true);
			if(!is_array($data) or !isset($data['followers_count'])){
				return false;
			}
			
			/* Return followers count */
			return $data['followers_count'];
			
		}
		
		/**
		 * Get YouTube channel subscriptions for given account
		 *
		 * Will be called by "SocialBox::refresh()"
		 */
		public static function refreshYoutube($youtubeId){
			
			/* Fetch data */
			$result = wp_remote_get(self::YOUTUBE_BASE . $youtubeId);
			
			/* check for unseccessful http requests */
			if(is_wp_error($result) or (wp_remote_retrieve_response_code($result) != 200)){
				return false;
			}
			
			/* Check for incorrect data */
			$data = simplexml_load_string(wp_remote_retrieve_body($result));
			if(!$data or isset($data->err) or !isset($data->children('http://gdata.youtube.com/schemas/2007')->statistics->attributes()->subscriberCount)){
				return false;
			}
			
			/* Return subscribers count */
			return (int) $data->children('http://gdata.youtube.com/schemas/2007')->statistics->attributes()->subscriberCount;
			
		}
		
		/**
		 * Get Feedburner subscriptions for given feed
		 *
		 * Will be called by "SocialBox::refresh()"
		 */
		public static function refreshFeedburner($feedburnerId){
			
			/* Fetch data */
			$result = wp_remote_get(self::FEEDBURNER_BASE . $feedburnerId);
			
			/* Check for unsuccessful http requests */
			if(is_wp_error($result) or (wp_remote_retrieve_response_code($result) != 200)){
				return false;
			}
			
			/* Check for incorrect data */
			$data = simplexml_load_string(wp_remote_retrieve_body($result));
			if(!$data or isset($data->err) or !isset($data->feed->entry['circulation'])){
				return false;
			}
			
			/* Return subscribers count */
			return (int) $data->feed->entry['circulation'];
			
		}
		
	}
	
	class SocialBoxWidget extends WP_Widget{
		
		/**
		 * The Widgets slug
		 */
		const SLUG = 'socialbox';
		
		/**
		 * Create a widget instance and set the base infos
		 */
		public function __construct(){
			
			/* Widget settings */
			$widgetOpts = array(
				'classname' => self::SLUG,
				'description' => __('Displays the numbers of Twitter Followers, Facebook Fans and Youtube Channel and Feedburner Feed Subscribers', self::SLUG)
			);
			
			/* Widget control settings */
			$controlOpts = array(
				'id_base' => self::SLUG
			);
			
			/* Create the widget */
			parent::__construct(self::SLUG, 'SocialBox', $widgetOpts, $controlOpts);
			
		}
		
		/**
		 * Display the actual Widget
		 *
		 * @param Array $args
		 * @param Array $instance
		 */
		public function widget($args, $instance){
			
			extract($args);
			
			/* Get current instances options */
			$facebookId = $instance['facebook_id'];
			$twitterId = $instance['twitter_id'];
			$youtubeId = $instance['youtube_id'];
			$feedburnerId = $instance['feedburner_id'];
			$showButtons = $instance['show_buttons'];
			
			/* Get counts from main plugin options */
			$opts = get_option(self::SLUG . '_options', null);
			if(!is_array($opts) or !isset($opts[$this->id])){
				$facebookCount = 0;
				$twitterCount = 0;
				$youtubeCount = 0;
				$feedburnerCount = 0;
			} else{
				$facebookCount = (isset($opts[$this->id]['facebook_count']) ? $opts[$this->id]['facebook_count'] : 0);
				$twitterCount = (isset($opts[$this->id]['twitter_count']) ? $opts[$this->id]['twitter_count'] : 0);
				$youtubeCount = (isset($opts[$this->id]['youtube_count']) ? $opts[$this->id]['youtube_count'] : 0);
				$feedburnerCount = (isset($opts[$this->id]['feedburner_count']) ? $opts[$this->id]['feedburner_count'] : 0);
			}
			
			/* Build links to network pages / feed */
			$facebook = "http://www.facebook.com/" . ((is_numeric($facebookId)) ? "profile.php?id={$facebookId}" : $facebookId);
			$twitter = "http://twitter.com/{$twitterId}";
			$youtube = "http://www.youtube.com/user/{$youtubeId}";
			$feedburner = "http://feeds.feedburner.com/{$feedburnerId}";
			
			/* Before Widget HTML */
			echo $before_widget;
				
			/* Social Box */
			include 'includes/widget.php';
			
			/* After Widget HTML */
			echo $after_widget;
			
			
		}
		
		/**
		 * Update Widget settings and refresh data for this Widget
		 *
		 * @param Array $newInstance
		 * @param Array $oldInstance
		 * @return Array
		 */
		public function update($newInstance, $oldInstance){
			
			/* Update widget settings */
			$instance = $oldInstance;
			$instance['facebook_id'] = trim(strip_tags($newInstance['facebook_id']));
			$instance['twitter_id'] = trim(strip_tags($newInstance['twitter_id']));
			$instance['youtube_id'] = trim(strip_tags($newInstance['youtube_id']));
			$instance['feedburner_id'] = trim(strip_tags($newInstance['feedburner_id']));
			$instance['show_buttons'] = $newInstance['show_buttons'];
			$instance['set_defaults'] = $newInstance['set_defaults'];
			$instance['facebook_default'] = ((is_numeric($newInstance['facebook_default'])) ? trim($newInstance['facebook_default']) : 0);
			$instance['twitter_default'] = ((is_numeric($newInstance['twitter_default'])) ? trim($newInstance['twitter_default']) : 0);
			$instance['youtube_default'] = ((is_numeric($newInstance['youtube_default'])) ? trim($newInstance['youtube_default']) : 0);
			$instance['feedburner_default'] = ((is_numeric($newInstance['feedburner_default'])) ? trim($newInstance['feedburner_default']) : 0);
			
			/* Save changes to global plugin options */
			$opts = get_option(self::SLUG . '_options', null);
			if(!is_array($opts)){
				$opts = array();
			}
			$opts[$this->id] = array(
				'facebook_id' => $instance['facebook_id'],
				'twitter_id' => $instance['twitter_id'],
				'youtube_id' => $instance['youtube_id'],
				'feedburner_id' => $instance['feedburner_id'],
				'set_defaults' => $instance['set_defaults'],
				'facebook_default' => $instance['facebook_default'],
				'twitter_default' => $instance['twitter_default'],
				'youtube_default' => $instance['youtube_default'],
				'feedburner_default' => $instance['feedburner_default']
				
			);
			update_option(self::SLUG . '_options', $opts);
			
			/* Force data refresh */
			SocialBox::refresh($this->id);
			
			return $instance;
			
		}
		
		/**
		 * Show the Widgets settings form
		 *
		 * @param Array $instance
		 */
		public function form($instance){
			
			/* Apply default values */
			$defaults = array(
				'facebook_id' => 	'',
				'twitter_id' => 	'',
				'youtube_id' => 	'',
				'feedburner_id' => 	'',
				'show_buttons' => 	true,
				'set_defaults' => false
			);
			$instance = wp_parse_args((array) $instance, $defaults);
			
			/* Show Widget Options */
			include 'includes/form.php';
			
		}
		
		/**
		 * Helper - Get absolute URL for $path
		 *
		 * @param String $path
		 * @return String
		 */
		private function getUrl($path){
			
			return plugins_url($path, __FILE__);
			
		}
		
		/**
		 * Helper - Echo absolute URL for $path
		 *
		 * @param String $path
		 */
		private function url($path){
			
			echo $this->getUrl($path);
			
		}
		
	}
	
}

/* Create instance of SocialBox*/
if(class_exists('SocialBox')){
	$SocialBox = new SocialBox();
}


